#!/usr/bin/env python
# coding: utf-8

# In[85]:

def find_line_intersection(p1, p2, p3, p4):
    '''
    Find the point of intersection between the lines p1 --> p2 and p3 --> p4.
    '''
    # Get the segments' parameters.
    dx12 = p2[0] - p1[0]
    dy12 = p2[1] - p1[1]
    dx34 = p4[0] - p3[0]
    dy34 = p4[1] - p3[1]

    # Solve for t1 and t2
    denominator = dy12 * dx34 - dx12 * dy34

    try:
        t1 = ((p1[0] - p3[0]) * dy34 + (p3[1] - p1[1]) * dx34) / denominator
        x = None
        print(x.test())
    except ZeroDivisionError:
        # The lines are parallel (or close enough to it).
        lines_intersect = False
        segments_intersect = False
        intersection = None
        close_p1 = None
        close_p2 = None
        return lines_intersect, segments_intersect, intersection, \
            close_p1, close_p2
    except Exception as ex:
        print(f'Unexpected error {type(ex).__name__}: {ex}')
        raise

    lines_intersect = True
    t2 = ((p3[0] - p1[0]) * dy12 + (p1[1] - p3[1]) * dx12) / -denominator

    # Find the point of intersection.
    intersection = (p1[0] + dx12 * t1, p1[1] + dy12 * t1)

    # The segments intersect if t1 and t2 are between 0 and 1.
    segments_intersect = ((t1 >= 0) and (t1 <= 1) and (t2 >= 0) and (t2 <= 1))

    # Find the closest points on the segments.
    if t1 < 0:
        t1 = 0
    elif t1 > 1:
        t1 = 1

    if t2 < 0:
        t2 = 0
    elif t2 > 1:
        t2 = 1

    close_p1 = (p1[0] + dx12 * t1, p1[1] + dy12 * t1)
    close_p2 = (p3[0] + dx34 * t2, p3[1] + dy34 * t2)

    return lines_intersect, segments_intersect, intersection, \
        close_p1, close_p2

# In[86]:

import tkinter as tk
import math

# Layout constants.
MARGIN = 5
WINDOW_WID = 400
WINDOW_HGT = 300

class CircleCircleApp:
    # Create and manage the tkinter interface.
    def __init__(self):
        # Make the main interface.
        self.window = tk.Tk()
        self.window.title('line_intersections')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry(f'{WINDOW_WID}x{WINDOW_HGT}')

        # Build the rest of the UI here...
        self.canvas = tk.Canvas(self.window, bg='white', cursor='cross',
            borderwidth=2, relief=tk.SUNKEN)
        self.canvas.pack(padx=10, pady=(0, 10),
            side=tk.BOTTOM, fill=tk.BOTH, expand=True)

        # Initially we have no lines.
        self.start_point = None
        self.end_point = None
        self.current_line = None

        # Make a list to hold the endpoints for two lines.
        self.points = []

        # Track Button-1 down, Button-1 motion, and Button-1 up.
        self.canvas.bind("<Button-1>", self.mouse_down)
        self.canvas.bind("<B1-Motion>", self.mouse_move)
        self.canvas.bind("<ButtonRelease-1>", self.mouse_up)

        # Display the window.
        self.window.focus_force()
        self.window.mainloop()

    def kill_callback(self):
        self.window.destroy()

    def mouse_down(self, event):
        '''Record the line's starting point.'''
        self.start_point = (event.x, event.y)

        # If we already have four points, start over.
        if len(self.points) == 4:
            self.points = []
            self.canvas.delete(tk.ALL)

    def mouse_move(self, event):
        # Ignore extraneous events.
        if self.start_point is None: return

        # If there is a current new line, remove it.
        if self.current_line is not None:
            self.canvas.delete(self.current_line)

        # Create the new temporary line.
        self.end_point = (event.x, event.y)
        self.current_line = self.canvas.create_line(
            self.start_point, self.end_point, fill='red')

    def mouse_up(self, event):
        # Ignore extraneous events.
        if self.start_point is None: return

        # If there is a current new line, remove it.
        if self.current_line is not None:
            self.canvas.delete(self.current_line)

        # Create the permanent line.
        self.canvas.create_line(
            self.start_point, self.end_point, fill='green')

        # Save the new line's data.
        self.points.append(self.start_point)
        self.points.append(self.end_point)

        # If we now have two lines, find their intersections.
        if len(self.points) == 4:
            # Get the intersection.
            lines_intersect, segments_intersect, intersection, \
                close_p1, close_p2 = find_line_intersection(*self.points)

            # Draw the closest segment.
            if lines_intersect:
               self.canvas.create_line(close_p1, close_p2, dash=(2, 4),
                                       fill='red')

            # Draw the points.
            for point in self.points:
                self.draw_point(point, 'white', 'black')

            # Draw the closest points.
            if lines_intersect:
                self.draw_point(close_p1, 'light blue', 'blue')
                self.draw_point(close_p2, 'light blue', 'blue')
                self.draw_point(intersection, 'hot pink', 'red')

            # We are no longer creating a line.
            self.start_point = None
            self.end_point = None
            self.current_line = None

    def draw_point(self, point, fill, outline):
        RADIUS = 3
        self.canvas.create_oval(
            point[0] - RADIUS, point[1] - RADIUS,
            point[0] + RADIUS, point[1] + RADIUS,
            fill=fill, outline=outline)

# In[87]:

CircleCircleApp()
print('Done')


# In[ ]:




